// Copyright (C) 2003 by Robert C. Martin and Micah D. Martin. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package fitnesse2;

import fitnesse2.http.*;
import fitnesse2.responders.*;
import fitnesse2.socketservice.SocketServer;
import fitnesse2.components.*;
import xpcinci.*;

import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.net.*;
import java.util.GregorianCalendar;

public class FitnesseServer implements SocketServer
{
	private ResponderFactory responderFactory;
	private Logger logger;

	public FitnesseServer(ResponderFactory responderFactory, Logger logger)
	{
		this.responderFactory = responderFactory;
		this.logger = logger;
	}

	public void serve(Socket s)
	{
		try
		{
			FitnesseResponseSender sender = new FitnesseResponseSender(s);
			HttpRequest request = sender.makeRequest();
			Response response = makeResponse(request);
			sender.sendResponse(response);
		}
		catch(SocketException se)
		{
			// can be thrown by makeResponse or sendResponse.
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	private Response makeResponse(HttpRequest request) throws Exception
	{
		Response response = null;
		try
		{
			request.parse();
			Responder responder = responderFactory.makeResponder(request);
			response = responder.makeResponse(request);
			response.addHeader("Server", "ECal-" + ECalendar.VERSION);
			response.addHeader("Connection", "close");
		}
		catch(SocketException se)
		{
			throw(se);
		}
		catch(Exception e)
		{
			response = new ErrorResponder(e).makeResponse(request);
		}
		return response;
	}

	public static LogData makeLogData(Socket socket, HttpRequest request, Response response)
	{
		LogData data = new LogData();
		data.host = ((InetSocketAddress) socket.getRemoteSocketAddress()).getAddress().getHostAddress();
		data.time = new GregorianCalendar();
		data.requestLine = request.getRequestLine();
		data.status = response.getStatus();
		data.size = response.getContentSize();

		return data;
	}

	public void log(Socket s, HttpRequest request, Response response) throws Exception
	{
		if(logger != null)
			logger.log(makeLogData(s, request, response));
	}

	private class FitnesseResponseSender implements ResponseSender
	{
		private Socket socket;
		private InputStream input;
		private OutputStream output;
		private HttpRequest request;
		private Response response;

		public FitnesseResponseSender(Socket s) throws Exception
		{
			socket = s;
			input = s.getInputStream();
			output = s.getOutputStream();
		}

		public void send(byte[] bytes) throws Exception
		{
			output.write(bytes);
			output.flush();
		}

		public void close() throws Exception
		{
			try
			{
				log(socket, request, response);
				socket.close();
			}
			catch(IOException e)
			{
				e.printStackTrace();
			}
		}

		public HttpRequest makeRequest() throws Exception
		{
			request = new HttpRequest(input);
			return request;
		}

		public void sendResponse(Response response) throws Exception
		{
			this.response = response;
			response.readyToSend(this);
		}
	}
}
