// Copyright (C) 2003 by Robert C. Martin and Micah D. Martin. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package fitnesse2;

import fitnesse2.testutil.*;
import fitnesse2.responders.*;
import fitnesse2.components.*;
import fitnesse2.http.*;
import junit.swingui.TestRunner;

import java.util.regex.Pattern;

public class FitnesseServerTest extends RegexTest
{
	public static void main(String[] args)
	{
		TestRunner.main(new String[]{"fitnesse.FitnesseServerTest"});
	}

	public void setUp() throws Exception
	{
	}

	public void tearDown() throws Exception
	{
	}

	public void testSimple() throws Exception
	{
		String output = getSocketOutput("GET /ECal HTTP/1.1\r\n\r\n");
		String statusLine = "HTTP/1.1 200 OK\r\n";
		assertTrue("Should have statusLine", Pattern.compile(statusLine, Pattern.MULTILINE).matcher(output).find());
		//assertTrue("Should have canned Content", hasSubString("some string", output));
	}

	
	public void testBadRequest() throws Exception
	{
		String output = getSocketOutput("Bad Request \r\n\r\n");

		String badRequest = "400 Bad Request";
		assertTrue("Should have baddRequest", hasSubString(badRequest, output));
	}


	public void testLoggingDataCreation() throws Exception
	{
		MockHttpRequest request = new MockHttpRequest();
		SimpleResponse response = new SimpleResponse(200);
		MockSocket socket = new MockSocket("something");

		socket.setHost("1.2.3.4");
		request.setRequestLine("GET / HTTP/1.1");
		response.setContent("abc");

		LogData data = FitnesseServer.makeLogData(socket, request, response);

		assertEquals("1.2.3.4", data.host);
		assertNotNull(data.time);
		assertEquals("GET / HTTP/1.1", data.requestLine);
		assertEquals(200, data.status);
		assertEquals(3, data.size);
	}
	
	public String getSocketOutput(String requestLine) throws Exception
	{
		MockSocket s = new MockSocket(requestLine);
		FitnesseServer server = new FitnesseServer(new ResponderFactory("."), null);
		server.serve(s);
		String output = s.getOutput();
		return output;
	}

	public String getSocketOutput() throws Exception
	{
		return getSocketOutput("GET /someResource HTTP/1.1\r\n\r\n");
	}

	private static boolean hasSubString(String expected, String output)
	{
		return Pattern.compile(expected, Pattern.MULTILINE).matcher(output).find();
	}
}
