// Copyright (C) 2003 by Robert C. Martin and Micah D. Martin. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package fitnesse2.http;

import java.nio.ByteBuffer;

public class ChunkedResponse extends Response
{
	private ResponseSender sender;
	private int bytesSent = 0;

	public void readyToSend(ResponseSender sender) throws Exception
	{
		this.sender = sender;

		addStandardHeaders();
		sender.send(makeHttpHeaders().getBytes());
	}

	protected void addSpecificHeaders()
	{
		addHeader("Transfer-Encoding", "chunked");
	}

	public static String asHex(int value)
	{
		return Integer.toHexString(value);
	}

	public void add(String text) throws Exception
	{
		add(text.getBytes());
	}

	public void add(byte[] bytes) throws Exception
	{
		String sizeLine = asHex(bytes.length) + CLRF;
		ByteBuffer chunk = ByteBuffer.allocate(sizeLine.length() + bytes.length + 2);
		chunk.put(sizeLine.getBytes()).put(bytes).put(CLRF.getBytes());
		sender.send(chunk.array());
		bytesSent += bytes.length;
	}

	public void done() throws Exception
	{
		sender.send( ("0" + CLRF + CLRF).getBytes() );
		sender.close();
	}

	public int getContentSize()
	{
		return bytesSent;
	}
}
