// Copyright (C) 2003 by Robert C. Martin and Micah D. Martin. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package fitnesse2.http;

import java.io.*;
import java.util.*;
import java.util.regex.*;

public class HttpResponseParser
{
	private int status;
	private String body;
	private HashMap headers = new HashMap();
	private BufferedReader input;

	private static final Pattern statusLinePattern = Pattern.compile("HTTP/\\d.\\d (\\d\\d\\d) ");
	private static final Pattern headerPattern = Pattern.compile("([^:]*): (.*)");

	public HttpResponseParser(InputStream input) throws Exception
	{
		this.input = new BufferedReader(new InputStreamReader(input));
		parseStatusLine();
		parseHeaders();
		parseBody();
	}

	private void parseStatusLine() throws Exception
	{
		String statusLine = input.readLine();
		Matcher match = statusLinePattern.matcher(statusLine);
		if(match.find())
		{
			String status = match.group(1);
			this.status = Integer.parseInt(status);
		}
		else
			throw new Exception("Could not parse Response");
	}

	private void parseHeaders() throws Exception
	{
		String line = input.readLine();
		while(!"".equals(line))
		{
			Matcher match = headerPattern.matcher(line);
			if(match.find())
			{
				String key = match.group(1);
				String value = match.group(2);
				headers.put(key, value);
			}
			line = input.readLine();
		}
	}

	private void parseBody() throws Exception
	{
		String lengthHeader = "Content-Length";
		if(hasHeader(lengthHeader))
		{
			int bytesToRead = Integer.parseInt(getHeader(lengthHeader));
			char[] buf = new char[bytesToRead];
			input.read(buf);
			body = new String(buf);
		}
	}

	public int getStatus()
	{
		return status;
	}

	public String getBody()
	{
		return body;
	}

	public String getHeader(String key)
	{
		return (String) headers.get(key);
	}

	public boolean hasHeader(String key)
	{
		return headers.containsKey(key);
	}

	public String toString()
	{
		StringBuffer buffer = new StringBuffer();
		buffer.append("Status: ").append(status).append("\n");
		buffer.append("Headers: ").append("\n");
		for(Iterator iterator = headers.keySet().iterator(); iterator.hasNext();)
		{
			String key = (String) iterator.next();
			buffer.append("\t").append(key).append(": ").append(headers.get(key)).append("\n");

		}
		buffer.append("Body: ").append("\n");
		buffer.append(body);
		return buffer.toString();
	}
}
