package ft.html;
import fit.*;

import java.net.*;
import java.util.*;
import java.util.regex.Pattern;
import com.gargoylesoftware.htmlunit.html.*;
import com.gargoylesoftware.htmlunit.*;

public class HtmlFixture extends Fixture {
	private static HashMap storage = new HashMap();
	public static HtmlPage htmlPage;

	public String htmlSource;
	public HtmlElement element;

	public void doTable(Parse table) {
		try {
			Parse sourceCell = getSourceCell(table);
			if (sourceCell == null) {
				Parse fixtureCell = table.parts.parts;
				ignore(fixtureCell);
				fixtureCell.addToBody(
					"<br>"
						+ label("The row naming the html source is missing"));
			} else {
				loadElement(sourceCell);
				if (element == null)
					missingElement(sourceCell, table);
				else
					doRows(table.parts.more.more);
			}
		} catch (Exception e) {
			exception(table.parts.parts, e);
		}
	}

	public void doRow(Parse row) {
		try {
			Parse cellOne = row.parts;
			String cellOneText = cellOne.text();
			if ("Elements".equals(cellOneText))
				cmd_elements(row);
			else if ("Element".equals(cellOneText))
				cmd_element(row);
			else if ("Attribute".equals(cellOneText))
				cmd_attribute(row);
			else if ("Text".equals(cellOneText))
				cmd_text(row);
			else if ("Has Text".equals(cellOneText))
				cmd_hasText(row);
			else if ("Matches Text".equals(cellOneText))
				cmd_matchesText(row);
			else if ("Execute".equals(cellOneText))
				cmd_execute(row);
			else {
				cellOne.addToBody(
					"<br>"
						+ label("Invalid token.  Valid tokens are:<br> Elements, Element, Attribute, Text, Has Text, Matches Text, Execute"));
				wrong(cellOne);
				ignoreTheRest(cellOne);
			}
		} catch (Exception e) {
			exception(row.parts, e);
		}
	}

	private void cmd_elements(Parse row) {
		Parse cellTwo = row.parts.more;
		String cellTwoText = cellTwo.text();

		int expected = Integer.parseInt(cellTwoText);
		int actual = element.getChildElements().size();

		if (actual == expected)
			right(cellTwo);
		else
			wrong(cellTwo, String.valueOf(actual));
	}

	private void cmd_element(Parse row) {
		Parse idCell = row.parts.more;
		Parse typeCell = idCell.more;
		Parse nameCell = typeCell.more;
		HtmlElement e = getElement(idCell);
		if (e != null) {
			right(idCell);
			checkElementType(typeCell, e);
			if (nameCell != null)
				storeElement(nameCell, e);
		} else {
			wrong(idCell);
			ignoreTheRest(idCell);
		}
	}

	private void cmd_matchesText(Parse row) {
		Parse textCell = row.parts.more;
		String expectedText = textCell.text();
		String actualText = element.asText();

		if (Pattern
			.compile(expectedText, Pattern.MULTILINE | Pattern.DOTALL)
			.matcher(actualText)
			.find())
			right(textCell);
		else {
			wrong(textCell);
			textCell.addToBody("<br>" + label("This pattern was not found"));
		}
	}

	private void cmd_hasText(Parse row) {
		Parse textCell = row.parts.more;
		String expectedText = textCell.text();
		String actualText = element.asText();

		if (actualText.indexOf(expectedText) != -1)
			right(textCell);
		else {
			wrong(textCell);
			textCell.addToBody("<br>" + label("This text was not found"));
		}
	}

	private void cmd_text(Parse row) {
		Parse textCell = row.parts.more;
		String expectedText = textCell.text();
		String actualText = element.asText();
		checkValues(expectedText, actualText, textCell);
	}

	private void cmd_attribute(Parse row) {
		Parse idCell = row.parts.more;
		Parse valueCell = idCell.more;

		String id = idCell.text();
		if (element.isAttributeDefined(id)) {
			right(idCell);
			if (valueCell != null) {
				String expectedValue = valueCell.text();
				String actrualValue = element.getAttributeValue(id);
				checkValues(expectedValue, actrualValue, valueCell);
			}
		} else {
			idCell.addToBody("<br>" + label("This attribute is not defined"));
			wrong(idCell);
			ignoreTheRest(idCell);
		}
	}

	private void cmd_execute(Parse row) {
		Parse attribCell = row.parts.more;
		String actionToRun = attribCell.text(); // (e.g. onClick)
		if (element.isAttributeDefined(actionToRun)) {
			String javaScriptCode = element.getAttributeValue(actionToRun);
			htmlPage.executeJavaScriptIfPossible(
				javaScriptCode,
				javaScriptCode,
				true,
				element);
		} else {
			attribCell.addToBody(
				"<br>" + label("This attribute is not defined"));
			wrong(attribCell);
			ignoreTheRest(attribCell);
		}
	}

	private Parse getSourceCell(Parse table) {
		Parse sourceRow = table.parts.more;
		if (sourceRow != null)
			return sourceRow.parts;
		else
			return null;
	}

	private void missingElement(Parse sourceCell, Parse table) {
		sourceCell.addToBody("<br>" + label("This element is not in storage"));
		wrong(sourceCell);
		Parse row = table.parts.more.more;
		while (row != null) {
			ignore(row.parts);
			ignoreTheRest(row.parts);
			row = row.more;
		}
	}

	private void loadElement(Parse sourceCell) {
		try {
			htmlSource = sourceCell.text();
			if (!isUrlValid(htmlSource))
				element = retrieve(htmlSource);
			else {
				determinePageAndElement();
			}
		} catch (Exception e) {
			exception(sourceCell.parts, e);
		}
	}

	void determinePageAndElement() throws Exception {
		WebClient webClient = new WebClient();
		URL url = new URL(htmlSource);
		htmlPage = (HtmlPage) webClient.getPage(url);
		element = htmlPage;
	}

	private HtmlElement getElement(Parse idCell) {
		HtmlElement e = null;
		String errorMessage = "";
		String id = idCell.text();
		try {
			e = element.getHtmlElementById(id);
		} catch (ElementNotFoundException e1) {
			errorMessage = "No element with this id";
		}

		if (e == null) {
			try {
				int index = Integer.parseInt(id);
				List children = element.getChildElements();
				if (children.size() >= index)
					e = (HtmlElement) children.get(index - 1);
				else
					errorMessage += " or index";
			} catch (NumberFormatException e1) {
				// id is not an index
			}
		}

		if (e == null)
			idCell.addToBody("<br>" + label(errorMessage));

		return e;
	}

	private void storeElement(Parse nameCell, HtmlElement e) {
		String name = nameCell.text();
		store(name, e);
		right(nameCell);
	}

	private void checkElementType(Parse typeCell, HtmlElement e) {
		String expectedType = typeCell.text();
		String actualType = e.getTagName();
		checkValues(expectedType, actualType, typeCell);
	}

	public static boolean isUrlValid(String url) {
		try {
			new URL(url);
			return true;
		} catch (MalformedURLException e) {
			return false;
		}
	}

	public static void store(String key, Object value) {
		storage.put(key, value);
	}

	public static HtmlElement retrieve(String key) {
		return (HtmlElement) storage.get(key);
	}

	private void checkValues(Object expected, Object actual, Parse parse) {
		if (expected.equals(actual))
			right(parse);
		else
			wrong(parse, actual.toString());
	}

	private void ignoreTheRest(Parse parse) {
		while (parse.more != null) {
			ignore(parse.more);
			parse = parse.more;
		}
	}
}
