package xpcinci.domain;

import java.util.*;
import java.util.GregorianCalendar;
import java.util.Calendar;

public class Week {
	int year;
	int weekNum;

	public Week(int year, int week) {
		this.year = year;
		this.weekNum = week;
	}

	public Week(Date date) {
		loadDate(date);
	}

	public int weekNum() {
		return weekNum;
	}

	private void loadDate(Date date) {
		GregorianCalendar cal = (GregorianCalendar) Calendar.getInstance();
		cal.setTime(date);
		weekNum = cal.get(GregorianCalendar.WEEK_OF_YEAR);
		year = cal.get(GregorianCalendar.YEAR);
		bumpYearBugFix(cal);
	}

	private void bumpYearBugFix(GregorianCalendar cal) {
		// weird fix for WeekTest.testAdd1WeekNewYear_2
		int doy = cal.get(GregorianCalendar.DAY_OF_YEAR);
		if (doy > 345 && weekNum == 1)
			year++;
	}

	public Week(String aString) {
		load(removeCommas(aString.trim()));
	}

	public Week(int gridvalue) {
		load(gridvalue);
	}

	private void load(String gridValue) {
		load((Integer.decode(gridValue)).intValue());
	}

	private void load(int gridValue) {
		final int maxweek = 999952;
		if (gridValue > maxweek) // toss day
			gridValue = gridValue / 100;
		if (gridValue > maxweek)
			throw new RuntimeException(
				"week value too big: " + (gridValue * 100));
		year = gridValue / 100;
		weekNum = gridValue % 100;
		if (weekNum == 54) {
			year++;
			weekNum = 1;
		}
		if (weekNum > 54)
			throw new RuntimeException("week value too big: " + weekNum);
	}

	public int[] getDayNumbers() {
		GregorianCalendar calendar = calendar();
		int[] dayNumbers = new int[7];
		for (int i = 0; i < dayNumbers.length; i++) {
			dayNumbers[i] = calendar.get(Calendar.DAY_OF_MONTH);
			calendar.add(Calendar.DATE, 1);
		}
		return dayNumbers;
	}

	public int getMonthOfSunday() {

		GregorianCalendar cal = calendar();
		return cal.get(Calendar.MONTH);
	}

	public int getMonthOfSaturday() {
		GregorianCalendar cal = calendar();
		cal.setFirstDayOfWeek(Calendar.SUNDAY);
		cal.set(Calendar.DAY_OF_WEEK, Calendar.SATURDAY);
		return cal.get(Calendar.MONTH);
	}

	public Week plus(int offset) {
		if (offset == 0)
			return this;
		Calendar c = calendar();
		c.add(GregorianCalendar.DATE, 7 * offset);
		return new Week(c.getTime());
	}

	public Iterator days() {
		List result = new ArrayList();
		Day day;
		result.add(day = sunday());
		result.add(day = day.tommorow());
		result.add(day = day.tommorow());
		result.add(day = day.tommorow());
		result.add(day = day.tommorow());
		result.add(day = day.tommorow());
		result.add(day = day.tommorow());
		return result.iterator();
	}

	public Day sunday() {
		Day day = new Day(calendar().getTime());
		return day;
	}

	public GregorianCalendar calendar() {
		GregorianCalendar calendar = new GregorianCalendar();
		calendar.setLenient(false);
		calendar.set(GregorianCalendar.YEAR, year);
		calendar.set(GregorianCalendar.WEEK_OF_YEAR, weekNum);
		calendar.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);
		return calendar;
	}

	public int month() {
		return calendar().get(GregorianCalendar.MONTH);
	}

	int week() {
		return weekNum;
	}

	int year() {
		return year;
	}

	public int value() {
		return (year * 100) + weekNum;
	}

	public String toString() {
		return getYear() + getWeek();
	}

	public String getWeek() {
		String weeknum = Integer.toString(weekNum);
		if (weeknum.length() < 2)
			weeknum = "0" + weeknum;
		return weeknum;
	}

	public String getYear() {
		return Integer.toString(year);
	}

	public Week nextWeek() {
		return new Week(value() + 1);
	}

	public Week prevWeek() {
		return new Week(value() - 1);
	}

	public boolean equals(Object other) {
		if (!(other instanceof Week))
			return false;
		return equals((Week) other);
	}

	public boolean equals(Week other) {
		if (value() == ((Week) other).value())
			return true;
		return false;
	}

	public boolean equals(String other) {
		return equals(new Week(other));
	}

	public boolean equals(int other) {
		return equals(new Week(other));
	}

	protected String removeCommas(String aString) {
		StringBuffer result = new StringBuffer();

		for (int i = 0; i < aString.length(); i++) {
			if (aString.charAt(i) != ',')
				result.append(aString.charAt(i));
		}
		return result.toString();
	}

}