package xpcinci.template;
import java.io.*;
import java.util.*;

import junit.framework.*;

public class FastTokenTest extends TestCase {

	StringWriter sw;
	PrintWriter pw;
	Map keys;

	public FastTokenTest(String arg0) {
		super(arg0);
	}

	protected void setUp() throws Exception {
		super.setUp();
		sw = new StringWriter();
		pw = new PrintWriter(sw);
		keys = new HashMap();
		keys.put("stuff", "LaLaLa");
		keys.put("two", "2222");
	}

	public void testBadValue() {
		keys.put("badtype", new Date());
		try {
			new FastToken("123<%badtype%>456", pw, keys);
			fail("Should throw exception");
		} catch (RuntimeException pe) {
			assertEquals(
				"Expecting String, found: java.util.Date",
				pe.getMessage());
		}
	}

	public void testParse() {
		new FastToken("123<%nostuff%>456", pw, keys);
		assertEquals("123<%nostuff%>456", sw.toString());
	}

	public void testParseStuff() {
		new FastToken("123<%stuff%>456", pw, keys);
		assertEquals("123LaLaLa456", sw.toString());
	}

	public void testBadParse1() {
		new FastToken("123<%stuff", pw, keys);
		assertEquals("123<%stuff", sw.toString());
	}

	public void testBadParse2() {
		new FastToken("123stuff%>aaa", pw, keys);
		assertEquals("123stuff%>aaa", sw.toString());
	}

	public void testParseWhiteSpace() {
		new FastToken("123<%         stuff     %>456", pw, keys);
		assertEquals("123LaLaLa456", sw.toString());
	}

	public void testParseTwo() {
		new FastToken("123<%stuff%>456<%two%>789", pw, keys);
		assertEquals("123LaLaLa4562222789", sw.toString());
	}

	public void testBadParseTwo() {
		new FastToken("123<%stuff%>456<%two789", pw, keys);
		assertEquals("123LaLaLa456<%two789", sw.toString());
	}

	public void testFilter() {
		new FastToken("123", pw, keys);
		assertEquals("123", sw.toString());
	}
}
