package xpcinci.time;

import java.util.*;

import junit.framework.*;

public class TEScriptTest extends TestCase {
    public TEScript script;
    
	public void testSimple() {
		script = new TEScript("(month 10)");
		assertTrue(script instanceof TEScript);

		TEScript script2 = new TEScript("(MONTH 12)");
		assertTrue(script2 instanceof TEScript);

		TEScript script3 = new TEScript("(dayname THU)");
		assertTrue(script3 instanceof TEScript);
	}

	// IDEA: Ignore Extra parens test
	//	public void testIgnoreParens() {
	//		TEScript script = new TEScript("((daynum 25) ((month 2)))");
	//		assertTrue(script.parse() instanceof TEScript);
	//		List l = script.contentNames();
	//		assertTrue(l.contains("daynum"));
	//	}

	public void testSimpleNumbers() {
		TEScript script = new TEScript("(daynum 25)");
		assertTrue(script instanceof TEScript);
	}

	public void testMissingOpen() {
		try {
			script = new TEScript("month 2)");
			fail("should throw");
		} catch (TEScriptException x) {
			assertEquals("Missing Open Paren", x.getMessage());
		}
	}

	public void testMissingClose() {
		try {
			script = new TEScript("(month 01");
			fail("should throw");
		} catch (TEScriptException x) {
			assertEquals("Missing Close Paren:  01", x.getMessage());
		}
	}

	public void testInvalidConst() {
		try {
			script = new TEScript("(day thorstag)");
			fail("should throw");
		} catch (TEScriptException x) {
			assertEquals("Invalid Keyword: [day]", x.getMessage());
		}
	}

	public void testIgnoreJunk() {
		TEScript script = new TEScript("junk1 (month 12) junk2");
		assertEquals("(month 12)", script.toString());
	}

	public void testBadContent() {
		try {
			script = new TEScript("(mon 12)");
			fail("should throw");
		} catch (TEScriptException x) {
			assertEquals("Invalid Keyword: [mon]", x.getMessage());
		}
	}

	public void testRecurse() {
		TEScript script = new TEScript("(time(month 2))");
		assertTrue(script instanceof TEScript);
	}

	public void testPeers() {
		TEScript script = new TEScript("(time (month 12)(daynum 25))");
		assertTrue(script instanceof TEScript);
		assertEquals("(time (month 12)(daynum 25))", script.toString());
	}

	public void testInvalidPeer() {
		try {
			script = new TEScript("(time (month 12)(DOC 25))");
			fail("should throw");
		} catch (TEScriptException x) {
			assertEquals("Invalid Keyword: [DOC]", x.getMessage());
		}
	}

	public void testInvalid() {
		try {
			script = new TEScript("(nothing)");
		} catch (TEScriptException tex) {
			return;
		} catch (Exception x) {
			fail("Should only throw a TEScriptException");
		}
	}

	public void testTotalTime() {
		TEScript s =
			new TEScript(
				"(time (year 2004)(month 12)(yearweek 200102)"
					+ "(daynum 35)(dayname tue)(hour 14)"
					+ "(min 34)(sec 45) (grid 2004527) (weeknum 200401) (date 20041231))");
		assertTrue(s instanceof TEScript);
	}

	public void testError() {
		TEScript s = new TEScript("(time (year 2003)(month 11)(daynum 12))");
		assertTrue(s instanceof TEScript);
	}

	public void testGetInt() {
		TEScript script = new TEScript("(month 12)(daynum 25)");
		assertEquals(12, script.getInt("month"));
		assertEquals(25, script.getInt("daynum"));

	}

	public void testError2() {
		TEScript script = new TEScript("(time( dow 03))");
		assertEquals(3, script.child.getInt("dow"));
	}

	public void testContentNames() {
		TEScript script = new TEScript("(time (month 12)(daynum 25))");
		assertTrue(script instanceof TEScript);
		List list = script.contentNames();
		assertEquals("month", list.get(0));
		assertEquals("daynum", list.get(1));
	}
}
